@@ -3,10 +3,8 @@
 
 #![cfg(test)]
 
-use {
-    std::convert::identity,
-    tests_utils::node_types::rc_pair::DatumAllocator,
-};
+use std::convert::identity;
+use tests_utils::node_types::rc_pair::DatumAllocator;
 
 tests_utils::eq_shapes_tests!(identity, DatumAllocator::new, identity,
                               #[ignore], #[ignore]);
