@@ -15,19 +15,19 @@ mod premade
             recursion::callstack::CallStack,
         },
         crate::{
+            anticipated_or_like::Infallible,
             generic::equiv::{
                 self,
                 Equiv,
             },
-            utils::IntoOk as _,
             Node,
         },
-        core::{
-            convert::Infallible,
-            marker::PhantomData,
-        },
+        core::marker::PhantomData,
     };
 
+    #[cfg(not(feature = "anticipate"))]
+    use crate::like_anticipated::IntoOk as _;
+
 
     /// Equivalence predicate that cannot handle cyclic nor very-deep graphs, and that has the
     /// least overhead.
@@ -52,6 +52,7 @@ mod premade
         }
 
         let mut e = Equiv::<Args<N>>::default();
+        #[allow(unstable_name_collisions)]
         e.equiv(a, b).into_ok()
     }
 
@@ -94,6 +95,7 @@ pub mod recursion
         use crate::{
             generic::equiv::{
                 self,
+                CounterpartsResult,
                 EdgesIter,
                 Equiv,
                 RecurMode,
@@ -121,17 +123,20 @@ pub mod recursion
                 edges_iter: EdgesIter<P::Node>,
             ) -> Result<<P::Node as Node>::Cmp, Self::Error>
             {
-                for (a, b) in edges_iter {
-                    match it.equiv_main(a, b) {
-                        Ok(cmp) if cmp.is_equiv() => (),
-                        result => return result,
+                for next in edges_iter {
+                    match next {
+                        Ok([a, b]) => match it.equiv_main(a, b) {
+                            Ok(cmp) if cmp.is_equiv() => (),
+                            result => return result,
+                        },
+                        Err(cmp_amount_edges) => return Ok(cmp_amount_edges),
                     }
                 }
                 Ok(Cmp::new_equiv())
             }
 
             #[inline]
-            fn next(&mut self) -> Option<(P::Node, P::Node)>
+            fn next(&mut self) -> Option<CounterpartsResult<P::Node>>
             {
                 None
             }
@@ -153,12 +158,12 @@ pub mod modes
     /// Do not limit the algorithm in how many nodes are traversed, and never abort early.
     pub mod unlimited
     {
-        use {
-            crate::generic::equiv::{
+        use crate::{
+            anticipated_or_like::Infallible,
+            generic::equiv::{
                 self,
                 DescendMode,
             },
-            core::convert::Infallible,
         };
 
         /// Specifies not limiting the amount of nodes traversed.
@@ -212,12 +217,13 @@ pub mod modes
         }
 
         pub(crate) use sealed::Ticker;
-        use {
-            crate::generic::equiv::{
+
+        use crate::{
+            anticipated_or_like::Infallible,
+            generic::equiv::{
                 self,
                 DescendMode,
             },
-            core::convert::Infallible,
         };
 
         /// Specifies limiting the amount of nodes traversed.  The inner value is the limit.
@@ -339,24 +345,15 @@ mod tests
             RefId(*self)
         }
 
-        fn amount_edges(&self) -> Self::Index
-        {
-            match self {
-                Datum::Leaf => 0,
-                Datum::Pair(_, _) => 2,
-            }
-        }
-
         fn get_edge(
             &self,
             idx: &Self::Index,
-        ) -> Self
+        ) -> Option<Self>
         {
             match (idx, self) {
-                #![allow(clippy::panic)]
-                (0, Datum::Pair(a, _)) => a,
-                (1, Datum::Pair(_, b)) => b,
-                _ => panic!("invalid"),
+                (0, Datum::Pair(a, _)) => Some(a),
+                (1, Datum::Pair(_, b)) => Some(b),
+                _ => None,
             }
         }
 
