@@ -3,11 +3,12 @@ pub use premade::*;
 mod premade
 {
     use {
-        super::recursion::vecstack::{
+        super::recursion::{
             self,
-            VecStack,
+            stack::RecurStack,
         },
         crate::{
+            anticipated_or_like::Infallible,
             basic::modes::{
                 limited::{
                     LimitReached,
@@ -20,15 +21,14 @@ mod premade
                 self,
                 Equiv,
             },
-            utils::IntoOk as _,
             Node,
         },
-        core::{
-            convert::Infallible,
-            marker::PhantomData,
-        },
+        core::marker::PhantomData,
     };
 
+    #[cfg(not(feature = "anticipate"))]
+    use crate::like_anticipated::IntoOk as _;
+
     /// Equivalence predicate that can handle very-wide graphs but not cyclic graphs.
     #[inline]
     pub fn equiv<N: Node>(
@@ -43,15 +43,16 @@ mod premade
             type DescendMode = Unlimited;
             type Error = Infallible;
             type Node = N;
-            type RecurMode = VecStack<Self>;
+            type RecurMode = RecurStack<Self>;
         }
 
-        impl<N: Node> vecstack::Params for Args<N>
+        impl<N: Node> recursion::stack::Params for Args<N>
         {
             type Node = N;
         }
 
         let mut e = Equiv::<Args<N>>::default();
+        #[allow(unstable_name_collisions)]
         e.equiv(a, b).into_ok()
     }
 
@@ -75,10 +76,10 @@ mod premade
             type DescendMode = Limited<L>;
             type Error = LimitReached;
             type Node = N;
-            type RecurMode = VecStack<Self>;
+            type RecurMode = RecurStack<Self>;
         }
 
-        impl<N: Node, L: Ticker> vecstack::Params for Args<N, L>
+        impl<N: Node, L: Ticker> recursion::stack::Params for Args<N, L>
         {
             type Node = N;
         }
@@ -92,7 +93,7 @@ mod premade
 /// Extend the algorithm to be able to traverse very-wide graphs.
 pub mod recursion
 {
-    pub mod vecstack
+    pub mod stack
     {
         //! Use [`Vec`] for the recursion stack, instead of the call-stack.
         //!
@@ -102,24 +103,30 @@ pub mod recursion
 
         use {
             crate::{
+                anticipated_or_like::Infallible,
                 basic::recursion::callstack::CallStack,
                 generic::equiv::{
                     self,
+                    CounterpartsResult,
                     EdgesIter,
                     Equiv,
                     RecurMode,
                 },
+                utils::{
+                    LazierIterator as _,
+                    LazyVecStack,
+                },
                 Cmp,
                 Node,
             },
             alloc::vec::Vec,
-            core::convert::Infallible,
         };
 
-        /// Generic parameters of [`VecStack`] and its operations.
+        /// Generic parameters of [`RecurStack`] and its operations.
         pub trait Params
         {
-            /// Amount of elements that a [`VecStack`] can contain initially before reallocating.
+            /// Amount of elements that a [`RecurStack`] can contain initially before
+            /// reallocating.
             ///
             /// An `impl` of [`Params`] may be made with a different value - either smaller or
             /// larger.  Note that the default only affects the initial capacity of the underlying
@@ -152,22 +159,23 @@ pub mod recursion
         /// (If, instead, you want to limit how much a recursion-stack can grow, you must `impl`
         /// [`RecurMode`] for your own type that does that and use it with the
         /// [`generic`](crate::generic) API.)
-        pub struct VecStack<P: Params>(Vec<EdgesIter<P::Node>>);
+        #[allow(clippy::module_name_repetitions)]
+        pub struct RecurStack<P: Params>(LazyVecStack<EdgesIter<P::Node>>);
 
-        impl<P: Params> Default for VecStack<P>
+        impl<P: Params> Default for RecurStack<P>
         {
             /// Create a new instance with capacity
             /// [`P::INITIAL_CAPACITY`](Params::INITIAL_CAPACITY).
             #[inline]
             fn default() -> Self
             {
-                Self(Vec::with_capacity(P::INITIAL_CAPACITY))
+                Self(LazyVecStack(Vec::with_capacity(P::INITIAL_CAPACITY)))
             }
         }
 
         /// Enables the call-stack to be used for the precheck and the vector-stack for the
         /// interleave, if desired.
-        impl<P: Params> From<CallStack> for VecStack<P>
+        impl<P: Params> From<CallStack> for RecurStack<P>
         {
             #[inline]
             fn from(_: CallStack) -> Self
@@ -176,8 +184,8 @@ pub mod recursion
             }
         }
 
-        /// Enables [`VecStack`] to be used with the algorithm.
-        impl<E, V> RecurMode<E> for VecStack<V>
+        /// Enables [`RecurStack`] to be used with the algorithm.
+        impl<E, V> RecurMode<E> for RecurStack<V>
         where
             E: equiv::Params<RecurMode = Self>,
             V: Params<Node = E::Node>,
@@ -191,34 +199,22 @@ pub mod recursion
                 edges_iter: EdgesIter<E::Node>,
             ) -> Result<<E::Node as Node>::Cmp, Self::Error>
             {
-                debug_assert!(!edges_iter.is_empty());
-                it.recur_mode.0.push(edges_iter);
+                it.recur_mode.0.extend(edges_iter);
                 Ok(Cmp::new_equiv())
             }
 
             #[inline]
-            fn next(&mut self) -> Option<(E::Node, E::Node)>
+            fn next(&mut self) -> Option<CounterpartsResult<E::Node>>
             {
-                if let Some(edges_iter) = self.0.last_mut() {
-                    let next = edges_iter.next();
-                    if edges_iter.is_empty() {
-                        // Prevent empty iterators from staying on the stack.
-                        drop(self.0.pop());
-                    }
-                    debug_assert!(next.is_some());
-                    next
-                }
-                else {
-                    None
-                }
+                self.0.next()
             }
 
-            /// An aborted precheck, that uses `VecStack`, might have left some elements, so we
-            /// must reset before doing the interleave using the same `VecStack`.
+            /// An aborted precheck, that uses `RecurStack`, might have left some elements, so we
+            /// must reset before doing the interleave using the same `RecurStack`.
             #[inline]
             fn reset(mut self) -> Self
             {
-                self.0.clear();
+                self.0.0.clear();
                 self
             }
         }
